/*
** Copyright (C) 2005 Antonin Stefanutti <antonin.stefanutti@alpha-sigma.net>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/*
   Convertor between 3DS Max ASE files and 3DFS Max (3D Flash Studio Max) XML files
*/

#include <cstdlib>
#include <stdio.h>
#include <fstream>
#include <string>
#include <iostream>
#include <sstream>
#include <math.h>

using namespace std;

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool getLine(ifstream& file, string s, istringstream& stream) {
	string line;
	stream.clear();
	while(file){
		getline(file, line);
		if (line.find(s,0) != string::npos) {
			stream.str(line);
			return true;
		}
	}
	return false;
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int round(float f) {
	return int((abs(f - floor(f)) < abs(f - ceil(f))) ? floor(f) : ceil(f));
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int main(int argc, const char **argv) {
	string mode;
	switch(argc) {
		case 3:
			mode = "tri";
			break;
		case 4:
			mode = argv[3];
			if ((mode != "tri") && (mode != "quad")){
				cout << "Error : invalid mode (tri or quad are supported)" << endl;
				cout << "Usage : 3dsMaxImport inputFile.ASE ouputFile.xml [mode = tri (|quad)]" << endl;
				return 1;
			}
			break;
		default :
			cout << "Usage : 3dsMaxImport inputFile.ASE ouputFile.xml [mode = tri (|quad)]" << endl;
			return 1;
	}
	string inputFileName = argv[1];
	string outputFileName = argv[2];
	ifstream inputFile(inputFileName.c_str());
	if (!inputFile) {
		cout << "Error opening file : " << inputFileName << " !" << endl;
		return 1;
	}
	ofstream outputFile(outputFileName.c_str());
	if (!outputFile) {
		cout << "Error creating file : " << outputFileName << " !" << endl;
		return 1;
	}
	istringstream streamLine;
	int numObject = 0;
	int numVertex = 0;
	int numFace = 0;
	string objectId;
	string str;
	outputFile << "<scene camera=\"400,0,0\" lookAt=\"0,0,0\" up=\"0,0,1\" angle=\"50\" width=\"443\" height=\"443\" near=\"0.1\" Ia=\"0.5\" Il=\"1.0\">" << endl;
	//_________________________________________________________________________________________________________________
	//try to find an object
	while (getLine(inputFile, "*GEOMOBJECT", streamLine)){
		outputFile << "<object";
		//_____________________________________________________________________________________________________________
		//try to find its name
		if (getLine(inputFile, "*NODE_NAME", streamLine)) {
			streamLine >> str;
			objectId = streamLine.str().substr(streamLine.str().find_first_of("\""),streamLine.str().find_last_of("\""));
			outputFile << " id=" << objectId;
		}
		else {
			outputFile << " id=" << "obj_" << numObject;
		}
		//_____________________________________________________________________________________________________________
		//try to find its wireframe color
		ostringstream color;
		ifstream::pos_type p = inputFile.tellg();
		if (getLine(inputFile, "*WIREFRAME_COLOR", streamLine)) {
			float R,G,B;
			streamLine >> str >> R >> G >> B;
			color << "0x" << hex << ((round(255*R) << 16) | (round(255*G) << 8) | round(255*B));
		}
		else {
			color << "0xAAAAAA";
		}
		inputFile.seekg(p);
		outputFile << " Ka=\"1.0\"" << " Kd=\"0.8\"" << " Ks=\"0.15\"" << /*" bfc=\"1\"" <<*/ " color=\"" << color.str() << "\"" << ">" << endl;
		//_____________________________________________________________________________________________________________
		//try to find its mesh
		if (getLine(inputFile, "*MESH", streamLine)) {
			//_________________________________________________________________________________________________________
			//try to find its number of vertices
			if (getLine(inputFile, "*MESH_NUMVERTEX", streamLine)) {
				streamLine >> str >> numVertex;
			}
			else {
				cout << "Error : no *MESH_NUMVERTEX found in *MESH !" << endl;
			}
			//_________________________________________________________________________________________________________
			//try to find its number of faces
			if (getLine(inputFile, "*MESH_NUMFACES", streamLine)) {
				streamLine >> str >> numFace;
			}
			else {
				cout << "Error : no *MESH_NUMFACES found in *MESH !" << endl;
			}
			//_________________________________________________________________________________________________________
			//try to find the vertices list
			if (getLine(inputFile, "*MESH_VERTEX_LIST", streamLine)) {
				outputFile << "<nodes>" << endl;
				string id,x,y,z;
				for (int i = 0; i < numVertex; i++) {
					if (getLine(inputFile, "*MESH_VERTEX", streamLine)) {
						streamLine >> str >> id >> x >> y >> z;
						outputFile << "<node id=\"" << id << "\" xyz=\"" << x << "," << y << "," << z << "\"/>" << endl;
					}
					else {
						cout << "Error : not enought *MESH_VERTEX found in *MESH_VERTEX_LIST !" << endl;
					}
				}
				outputFile << "</nodes>" << endl;
			}
			else {
				cout << "Error : no *MESH_VERTEX_LIST found in *MESH !" << endl;
			}
			//_________________________________________________________________________________________________________
			//try to find the faces list
			if (getLine(inputFile, "*MESH_FACE_LIST", streamLine)) {
				outputFile << "<polygons>" << endl;
				if (mode == "tri"){
					string id,v0,v1,v2;
					for (int i = 0; i < numFace; i++) {
						if (getLine(inputFile, "*MESH_FACE",streamLine)) {
							streamLine >> str >> id >> str >> v0 >> str >> v1 >> str >> v2;
							id.replace(id.find_last_of(":"),1,"");
							outputFile << "<polygon id=\"" << id << "\">" << endl;
							outputFile << "<node id=\"" << v2 << "\"/>" << endl;
							outputFile << "<node id=\"" << v1 << "\"/>" << endl;
							outputFile << "<node id=\"" << v0 << "\"/>" << endl;
							outputFile << "</polygon>" << endl;
						}
						else {
							cout << "Error : not enought *MESH_FACE found in *MESH_FACE_LIST !" << endl;
						}
					}
				}
				else if (mode == "quad") {
					string id0,id1;
					string v0[3],v1[3];
					string e0[3],e1[3];
					string pt[4];
					int i = 0;
					while (i < numFace) {
						//_____________________________________________________________________________________________
						//try to find the face
						if (getLine(inputFile, "*MESH_FACE",streamLine)) {
							streamLine >> str >> id0 >> str >> v0[0] >> str >> v0[1] >> str >> v0[2] >> str >> e0[0] >> str >> e0[1] >> str >> e0[2];
							if (getLine(inputFile, "*MESH_FACE",streamLine)) {
								id0.replace(id0.find_last_of(":"),1,"");
								streamLine >> str >> id1 >> str >> v1[0] >> str >> v1[1] >> str >> v1[2] >> str >> e1[0] >> str >> e1[1] >> str >> e1[2];
								outputFile << "<polygon id=\"" << id0 << "\">" << endl;
								if (e0[0] == "1"){
									pt[0] = v0[0];
									pt[1] = v0[1];
								}
								else if (e0[1] == "1"){
									pt[0] = v0[1];
									pt[1] = v0[2];
								}
								else if (e0[2] == "1"){
									pt[0] = v0[2];
									pt[1] = v0[0];
								}
								if (e1[0] == "1"){
									pt[2] = v1[0];
									pt[3] = v1[1];
								}
								else if (e1[1] == "1"){
									pt[2] = v1[1];
									pt[3] = v1[2];
								}
								else if (e1[2] == "1"){
									pt[2] = v1[2];
									pt[3] = v1[0];
								}
								outputFile << "<node id=\"" << pt[3] << "\"/>" << endl;
								outputFile << "<node id=\"" << pt[2] << "\"/>" << endl;
								outputFile << "<node id=\"" << pt[1] << "\"/>" << endl;
								outputFile << "<node id=\"" << pt[0] << "\"/>" << endl;
								outputFile << "</polygon>" << endl;
							}
							else {
								cout << "Error : not enought *MESH_FACE found in *MESH_FACE_LIST !" << endl;
							}
						}
						else {
							cout << "Error : not enought *MESH_FACE found in *MESH_FACE_LIST !" << endl;
						}
						i += 2;
					}
				}
				outputFile << "</polygons>" << endl;
			}
			else {
				cout << "Error : no *MESH_VERTEX_LIST found in *MESH !" << endl;
			}
			//_________________________________________________________________________________________________________
		}
		else {
			cout << "Error : no *MESH found in *GEOMOBJECT !" << endl;
		}
		//_____________________________________________________________________________________________________________
		outputFile << "</object>" << endl;
		numObject++;
	}
	if (numObject == 0) {
		cout << "Error : no *GEOMOBJECT found in "  << inputFileName << "!" << endl;
	}
	outputFile << "</scene>" << endl;
	inputFile.close();
	outputFile.close();
	return 0;
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

